/*
 * Copyright (c) 2017 Gregory Penn.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#ifndef GDAL_LAYERFEATURE_HPP
#define GDAL_LAYERFEATURE_HPP

#include <ogr_feature.h>
#include <gdal/smartptr.hpp>

namespace gdal
{

  class layerfeature
  {
    using feature_ptr = smartptr<OGRFeature>;

  public:
    layerfeature (feature_ptr p): p_(p) {}

    auto field_as_string (int index)
    {
      std::string result(p_->GetFieldAsString(index));
      return result;
    }

    auto field_as_string (const char* name)
    {
      std::string result(p_->GetFieldAsString(name));
      return result;
    }

    auto field_as_double (int index)
    {
      double result(p_->GetFieldAsDouble(index));
      return result;
    }

    auto field_as_double (const char* name)
    {
      double result(p_->GetFieldAsDouble(name));
      return result;
    }

    auto field_as_integer (int index)
    {
      double result(p_->GetFieldAsInteger(index));
      return result;
    }

    auto field_as_integer (const char* name)
    {
      double result(p_->GetFieldAsInteger(name));
      return result;
    }

    auto definition () { return p_->GetDefnRef(); }
    auto FID () { return p_->GetFID(); }

    auto gdal_ptr () { return p_; }

  private:
    auto p () { return p_; }

  private:
    feature_ptr p_;
  };

} // namespace gdal

#endif
