/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_CONCEPT_REFERENCE_WRAPPER_CONCEPT_HPP
#define GRAPH_MODEL_CONCEPT_REFERENCE_WRAPPER_CONCEPT_HPP 1

#include <boost/concept_check.hpp>
#include <boost/hana/traits.hpp>

namespace graph_model
{

  namespace concept
  {

    /// \brief Reference wrapper concept check
    /// \tparam T The type to check
    ///
    /// The class checks that a type \c T models the Reference Wrapper
    /// concept.  Specifically, this checks for the \c type associated
    /// type and the \c get() member function, which should return a
    /// reference to the wrapped type.  Note that reference wrappers
    /// also provide a function call member function, but this will
    /// not be applicable unless the wrapped type also does so.
    /// Consequently, it is not checked for here.
    ///
    template < typename T >
    class reference_wrapper_concept
    {
      using type = typename T::type;

    public:
      BOOST_CONCEPT_USAGE(reference_wrapper_concept)
      {
	type& wrapped_ = t_.get();
      }

    private:
      T t_;
    };

  } // namespace concept

} // namespace graph_model

#endif
