/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_GRAPH_INCIDENCE_GRAPH_BASE_HPP
#define GRAPH_MODEL_GRAPH_INCIDENCE_GRAPH_BASE_HPP 1

#include <graph_model/graph/graph_base.hpp>
#include <graph_model/graph/streamable_base.hpp>
#include <graph_model/graph_tags.hpp>
#include <graph_model/traits/vertex_descriptor.hpp>

namespace graph_model
{

  namespace graph
  {

    template < typename Derived, typename Base = graph_base >
    class incidence_graph_base
      : public Base
    {
      using vertex_descriptor = traits::vertex_descriptor_t<::graph_model::tag::incidence_graph>;

    public:
      incidence_graph_base () = default;

      template < typename Graph >
      incidence_graph_base (Graph const& g, vertex_descriptor source)
	: Base(g,source)
	, source_(source)
      {}

      auto source () const { return source_; }

    private:
      vertex_descriptor source_;
    };

    template < typename VertexDescriptor, typename Derived, typename Base >
    auto out_degree (VertexDescriptor u, incidence_graph_base<Derived,Base> const&)
    { return u.out_degree(); }

    template < typename VertexDescriptor, typename Derived, typename Base >
    auto out_edges (VertexDescriptor u, incidence_graph_base<Derived,Base> const&)
    { return u.out_edges(); }

    template < typename EdgeDescriptor, typename Derived, typename Base >
    auto source (EdgeDescriptor e, incidence_graph_base<Derived,Base> const&)
    { return e.source(); }

    template < typename EdgeDescriptor, typename Derived, typename Base >
    auto target (EdgeDescriptor e, incidence_graph_base<Derived,Base> const&)
    { return e.target(); }

  } // namespace graph

} // namespace graph_model

#endif
