/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_TAGS_HPP
#define GRAPH_MODEL_VERTEX_TAGS_HPP 1

#include <iostream>

namespace graph_model
{

  namespace vertex
  {

    /// \brief Tags to define types of vertices
    ///
    /// Vertex tags are organized into a class hierarchy that reflects
    /// the class hierarchy of the vertex classes themselves.  For
    /// example, all are derived from vertex::tag::vertex.  This
    /// enables tag dispatching based upon the category of a vertex.
    ///
    namespace tag
    {

      struct vertex {};

      struct expression : vertex {};

      struct factor : expression {};
      struct scalar_factor : factor {};

      struct distribution : factor {};
      struct scalar_distribution : distribution {};

      struct function {};

      struct random_variable : expression {};
      struct scalar_random_variable : random_variable {};
      struct summarized_random_variable : random_variable {};
      struct summarized_scalar_random_variable : scalar_random_variable {};

      struct MH_proposer : vertex {};
      struct basic_MH_proposer : MH_proposer {};

      struct sampler : vertex {};
      struct basic_sampler : sampler {};

#define STREAMABLE_TAG(Tag)				\
      std::ostream& operator << (std::ostream& os, Tag) \
      { return os << #Tag; }				\
      /**/

      STREAMABLE_TAG(MH_proposer)
      STREAMABLE_TAG(distribution)
      STREAMABLE_TAG(expression)
      STREAMABLE_TAG(factor)
      STREAMABLE_TAG(function)
      STREAMABLE_TAG(random_variable)
      STREAMABLE_TAG(sampler)
      STREAMABLE_TAG(scalar_distribution)
      STREAMABLE_TAG(scalar_factor)
      STREAMABLE_TAG(scalar_random_variable)
      STREAMABLE_TAG(summarized_random_variable)
      STREAMABLE_TAG(summarized_scalar_random_variable)
      STREAMABLE_TAG(vertex)

#undef STREAMABLE_TAG

    } // namespace tag

  } // namespace vertex

} // namespace graph_model

#endif
