/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef TRAITS_VECTOR_HPP
#define TRAITS_VECTOR_HPP 1

#include <vector>
#include <traits/convert.hpp>
#include <traits/domain.hpp>
#include <traits/probability.hpp>
#include <traits/real.hpp>
#include <traits/value.hpp>
#include <traits/variate.hpp>
#include <type_traits/common_type.hpp>

namespace traits
{

  namespace defaults
  {

    template < typename T, typename Alloc, typename U >
    struct convert< std::vector<T,Alloc>,U >
    {
      using type = std::vector<U,typename Alloc::template rebind<U>::other>;
    };

    template < typename T, typename Alloc, typename U >
    struct convert< std::vector<T,Alloc> const,U >
    {
      using type = std::vector<U,typename Alloc::template rebind<U>::other> const;
    };

    template < typename T, typename Alloc >
    struct domain< std::vector<T,Alloc>,std::false_type >
      : domain<T>
    {};

    template < typename T, typename Alloc >
    struct probability< std::vector<T,Alloc> >
      : probability<T>
    {};

    template < typename T, typename Alloc >
    struct real< std::vector<T,Alloc>,std::false_type,std::false_type >
      : real<T>
    {};

    template < typename T, typename Alloc >
    struct value< std::vector<T,Alloc> >
    {
      using type = T;
    };

    template < typename T, typename Alloc >
    struct variate< std::vector<T,Alloc> >
    {
      using type = std::vector<T,Alloc>;
    };

  } // namespace defaults

} // namespace traits

namespace type_traits
{

  namespace defaults
  {

    template < typename T, typename AllocT, typename U, typename AllocU >
    class common_type< std::vector<T,AllocT>,std::vector<U,AllocU> >
    {
      using V = std::vector<common_type_t<T,U>>;
      using AllocV = typename AllocT::template rebind<V>::other;
    public:
      using type = std::vector<V,AllocV>;
    };

  } // namespace defaults

} // namespace type_traits

#endif
