/*
 * Copyright (c) 2017 Gregory Penn.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#define BOOST_TEST_MODULE gdal

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <iostream>
#include <ogr_core.h>
#include <gdal/geographic_srs.hpp>
#include <gdal/OGRCoordinateTransformation_handle.hpp>
using std::cout;
using std::endl;
using namespace gdal;

BOOST_AUTO_TEST_CASE(constructor)
{
  auto wgs84 = make_wgs84();
  auto epsg32613 = make_geographic_srs(32613);
  auto transformer = OGRCoordinateTransformation_handle(wgs84, epsg32613);
}

BOOST_AUTO_TEST_CASE(function_call_operator_map)
{
  using std::string;

  auto wgs84 = make_wgs84();
  auto epsg32613 = make_geographic_srs(32613);
  auto transformer = OGRCoordinateTransformation_handle(wgs84, epsg32613);

  std::map<string, double> coords_wgs84 {{"x", -106.58804}, {"y", 32.20300}};
  auto transformed = transformer(coords_wgs84);
}

BOOST_AUTO_TEST_CASE(function_call_operator_vec)
{
  auto wgs84 = make_wgs84();
  auto epsg32613 = make_geographic_srs(32613);
  auto transformer = OGRCoordinateTransformation_handle(wgs84, epsg32613);

  std::vector<double> coords_wgs84 { -106.58804, 32.20300 };
  auto transformed = transformer(coords_wgs84);
}
