/*
 * Copyright (c) 2017 Gregory Penn.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#define BOOST_TEST_MODULE gdal

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <iostream>
#include <boost/mpl/vector.hpp>
#include <iostream>
#include <gdal/rasterdataset.hpp>

typedef boost::mpl::vector<int, double, float> value_types;

using std::cout;
using std::endl;
using namespace gdal;

#define rasterfile "../data/test_raster.tif"

BOOST_AUTO_TEST_CASE_TEMPLATE(raster_default_constructor_value_types,T,value_types)
{
  rasterdataset<T> h;
}

BOOST_AUTO_TEST_CASE_TEMPLATE(raster_datasource_constructor_value_types_updateable,T,value_types)
{
  rasterdataset<T> h(rasterfile, true);
  BOOST_ASSERT(h.updatable() == true);
  h.srs();
  h.grid();
}

BOOST_AUTO_TEST_CASE_TEMPLATE(raster_datasource_constructor_value_types_readonly,T,value_types)
{
  rasterdataset<T> h(rasterfile, false);
  BOOST_ASSERT(h.updatable() == false);
  h.srs();
  h.grid();
}

BOOST_AUTO_TEST_CASE_TEMPLATE(raster_newraster_constructor_value_types_vector_coeffs,T,value_types)
{
  std::string name = "newraster.tif";
  int xsize = 2;
  int ysize = 3;
  int nbands = 1;
  auto srs = make_geographic_srs(3857);
  auto coeffs = std::vector<double>({ -11866500, 1000, 0, 3790500, 0, -500 });
  rasterdataset<T> h(name, xsize, ysize, nbands, srs, coeffs);
  h.srs();
  h.grid();
}

BOOST_AUTO_TEST_CASE_TEMPLATE(raster_newraster_constructor_value_types_stdarray_coeffs,T,value_types)
{
  std::string name = "newraster.tif";
  int xsize = 2;
  int ysize = 3;
  int nbands = 1;
  auto srs = make_geographic_srs(3857);
  auto coeffs = std::array<double, 6>({ -11866500, 1000, 0, 3790500, 0, -500 });
  rasterdataset<T> h(name, xsize, ysize, nbands, srs, coeffs);
  h.srs();
  h.grid();

}

BOOST_AUTO_TEST_CASE_TEMPLATE(raster_band_value_types,T,value_types)
{
  auto band1 = rasterdataset<T>(rasterfile, true ).band(1);
  auto band2 = rasterdataset<T>(rasterfile, false).band(1);
}
