/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MODULE graph_model_distribution_multinomial_multinomial_distribution

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <graph_model/distribution/multinomial/multinomial_distribution.hpp>

#include <graph_model/distribution/parameter/mean.hpp>
#include <graph_model/distribution/parameter/precision.hpp>
#include <graph_model/distribution/parameter/probability.hpp>
#include <graph_model/distribution/parameter/variate.hpp>

#include <sample_space/vector/simplex.hpp>
#include <sample_space/vector/vector.hpp>

BOOST_AUTO_TEST_CASE(test_constructor)
{
  using simplex_type = sample_space::vector::simplex<double>;
  using vector_type = sample_space::vector::vector<double>;

  using graph_model::distribution::multinomial::multinomial_distribution;

  namespace parameter = graph_model::distribution::parameter;

  simplex_type variate (3);
  variate.at(2) = 0.1;
  variate.at(1) = 0.2;
  variate.at(0) = 0.7;

  vector_type probability (3);
  probability.at(2) = 1;
  probability.at(1) = 1;
  probability.at(0) = 1;

  { auto distribution = multinomial_distribution(); }
  { auto distribution = multinomial_distribution(probability); }
  { auto distribution = multinomial_distribution(probability,variate); }
  { auto distribution = multinomial_distribution(parameter::probability=probability); }
  { auto distribution = multinomial_distribution(parameter::probability=probability,variate); }
  { auto distribution = multinomial_distribution(variate,parameter::probability=probability); }
  { auto distribution = multinomial_distribution(parameter::probability=probability); }
  { auto distribution = multinomial_distribution(parameter::probability=probability,parameter::variate=variate); }
}
