#
# Copyright (c) 2017 Brook Milligan.
#
# This file is part of the GraphModel Library.
#
#     The GraphModel Library is free software: you can redistribute it
#     and/or modify it under the terms of the GNU Affero General
#     Public License as published by the Free Software Foundation,
#     either version 3 of the License, or (at your option) any later
#     version.
#
#     The GraphModel Library is distributed in the hope that it will
#     be useful, but WITHOUT ANY WARRANTY; without even the implied
#     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#     See the GNU Affero General Public License for more details.
#
#     You should have received a copy of the GNU Affero General Public
#     License along with the GraphModel Library.  If not, see
#     <http://www.gnu.org/licenses/>.
#

library(dplyr)
library(ggplot2)

sample.size <- 20
b0 <- -5
b1 <-  0.8
xmin <- 0
xmax <- 10

filename <- 'samples.dat'

logistic <- function (x) { 1/(1 + exp(-x)) }
linear_predictor <- function (b0, b1, x) { b0 + b1*x }
bernoulli <- function (p) { rbinom(length(p),1,p) }
model_y <- function (x) { linear_predictor(b0, b1, x) %>% logistic() %>% bernoulli() }

x <- runif(sample.size, xmin, xmax)
p <- logistic(linear_predictor(b0, b1, x))
observed <- bernoulli(p)

samples.df <- data_frame(x, observed, p) %>% arrange(p)
model <- glm(observed ~ x, family = binomial(link='logit'), data = samples.df)
b0_est <- coef(model)[[1]]
b1_est <- coef(model)[[2]]

# ggplot(samples.df, aes(x=x, y=observed)) +
#   stat_function(fun = function (x) { logistic(linear_predictor(b0, b1, x)) }, alpha = 0.4) +
#   geom_point(data = samples.df, aes(y=observed)) +
#   # geom_segment(aes(xend=x, yend=p), alpha=0.2) +
#   geom_smooth(method = "glm", method.args = list(family = binomial(link='logit')), se = FALSE) +
#   ylab("p")

write.table(samples.df,filename,quote=FALSE,sep="\t",row.names=FALSE)

str(samples.df)
print(paste('samples:',sample.size))
print(paste('mean:', mean(samples.df$observed)))
print(paste('b0 actual:', b0))
print(paste('b0 est   :', b0_est))
print(paste('b1 actual:', b1))
print(paste('b1 est   :', b1_est))
