/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MODULE graph_model_vertex_distribution

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <graph_model/concept/distribution_archetype.hpp>
#include <graph_model/concept/distribution_concept.hpp>
#include <graph_model/concept/distribution_policy_archetype.hpp>
#include <graph_model/vertex/distribution.hpp>

#include <boost/mpl/pair.hpp>
#include <boost/mpl/vector.hpp>

template < typename Variate, typename Probability >
struct distribution_policy
{
  template < typename ... T >
  auto sample (T&& ...) const { return Variate{42}; }
  template < typename ... T >
  auto pdf (T&& ...) const { return Probability{0.5}; }
};

using test_types = boost::mpl::vector
  <
  boost::mpl::pair<int,double>
  , boost::mpl::pair<long,double>
  , boost::mpl::pair<float,double>
  , boost::mpl::pair<double,double>
  , boost::mpl::pair<long double,double>
  >;

BOOST_AUTO_TEST_CASE(test_distribution_archetype)
{
  using distribution_type = graph_model::concept::distribution_archetype<>;
  BOOST_CONCEPT_ASSERT((graph_model::concept::distribution_concept<distribution_type>));
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_default_constructor,Pair,test_types)
{
  using variate_type = typename Pair::first;
  using probability_type = typename Pair::second;
  using distribution_type = graph_model::vertex::distribution<variate_type,probability_type>;
  distribution_type expr = {};
#if XXX_UNDEFINED_BEHAVIOR
  expr();
  expr.eval();
  expr.sample();
  expr.pdf();
#endif	// XXX_UNDEFINED_BEHAVIOR
  distribution_type other = std::move(expr);
}

#if XXX_IMPLICITLY_DELETED_COPY_CONSTRUCTOR
BOOST_AUTO_TEST_CASE_TEMPLATE(test_copy_constructor,Pair,test_types)
{
  using variate_type = typename Pair::first;
  using probability_type = typename Pair::second;
  using distribution_type = graph_model::vertex::distribution<variate_type,probability_type>;
  distribution_type expr = {};
  distribution_type other { expr };
}
#endif	// XXX_IMPLICITLY_DELETED_COPY_CONSTRUCTOR

BOOST_AUTO_TEST_CASE_TEMPLATE(test_move_constructor,Pair,test_types)
{
  using variate_type = typename Pair::first;
  using probability_type = typename Pair::second;
  using distribution_type = graph_model::vertex::distribution<variate_type,probability_type>;
  distribution_type expr = {};
  distribution_type other = std::move(expr);
}

#if XXX_IMPLICITLY_DELETED_COPY_ASSIGNMENT
BOOST_AUTO_TEST_CASE_TEMPLATE(test_copy_assignment,Pair,test_types)
{
  using variate_type = typename Pair::first;
  using probability_type = typename Pair::second;
  using distribution_type = graph_model::vertex::distribution<variate_type,probability_type>;
  distribution_type expr = {};
  distribution_type other = {};
  other = expr;
}
#endif	// XXX_IMPLICITLY_DELETED_COPY_ASSIGNMENT

BOOST_AUTO_TEST_CASE_TEMPLATE(test_move_assignment,Pair,test_types)
{
  using variate_type = typename Pair::first;
  using probability_type = typename Pair::second;
  using distribution_type = graph_model::vertex::distribution<variate_type,probability_type>;
  distribution_type expr = {};
  distribution_type other = {};
  other = std::move(expr);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_reset_strategy,Pair,test_types)
{
  using variate_type = typename Pair::first;
  using probability_type = typename Pair::second;
  using distribution_type = graph_model::vertex::distribution<variate_type,probability_type>;
  distribution_type expr = {};
  expr.reset_strategy(graph_model::concept::distribution_policy_archetype<variate_type,probability_type>());
#if XXX_UNDEFINED_BEHAVIOR
  expr();
  expr.eval();
  expr.sample();
  expr.pdf();
#endif	// XXX_UNDEFINED_BEHAVIOR
  expr.reset_strategy(distribution_policy<variate_type,probability_type>());
  BOOST_CHECK_EQUAL(expr(),0.5);
  BOOST_CHECK_EQUAL(expr.eval(),0.5);
  BOOST_CHECK_EQUAL(expr.sample(),42);
  BOOST_CHECK_EQUAL(expr.pdf(),0.5);
}
