/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#define BOOST_TEST_MODULE sample_space_random_multinomial_distribution

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <sample_space/random/multinomial_distribution.hpp>
#include <sample_space/vector/simplex.hpp>
#include <sample_space/vector/vector.hpp>
#include <boost/random/mersenne_twister.hpp>

#include <boost/mpl/pair.hpp>
#include <boost/mpl/vector.hpp>

using test_types = boost::mpl::vector<
  boost::mpl::pair<std::vector<float>,std::vector<int>>
  , boost::mpl::pair<std::vector<float>,std::vector<long>>
  , boost::mpl::pair<std::vector<double>,std::vector<int>>
  , boost::mpl::pair<std::vector<double>,std::vector<long>>
  , boost::mpl::pair<std::vector<long double>,std::vector<long long>>
  , boost::mpl::pair<sample_space::vector::simplex<float>,sample_space::vector::vector<int>>
  , boost::mpl::pair<sample_space::vector::simplex<double>,sample_space::vector::vector<long>>
  , boost::mpl::pair<sample_space::vector::simplex<long double>,sample_space::vector::vector<long long>>
  >;

BOOST_AUTO_TEST_CASE_TEMPLATE(test_default_constructor,Pair,test_types)
{
  using probability_vector_type = typename Pair::first;
  using probability_type = typename probability_vector_type::value_type;
  using size_vector_type = typename Pair::second;
  using size_type = typename size_vector_type::value_type;

  using distribution_type = sample_space::random::multinomial_distribution<size_vector_type,probability_vector_type>;

  distribution_type distribution;
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_initializer_list_constructor,Pair,test_types)
{
  using probability_vector_type = typename Pair::first;
  using probability_type = typename probability_vector_type::value_type;
  using size_vector_type = typename Pair::second;
  using size_type = typename size_vector_type::value_type;

  using distribution_type = sample_space::random::multinomial_distribution<size_vector_type,probability_vector_type>;

  using T = probability_type;
  distribution_type distribution { T(0.1), T(0.2), T(0.3), T(0.4) };
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_sample,Pair,test_types)
{
  using probability_vector_type = typename Pair::first;
  using probability_type = typename probability_vector_type::value_type;
  using size_vector_type = typename Pair::second;
  using size_type = typename size_vector_type::value_type;

  using distribution_type = sample_space::random::multinomial_distribution<size_vector_type,probability_vector_type>;

  using generator_type = boost::mt19937;

  using T = probability_type;
  distribution_type distribution { T(0.1), T(0.2), T(0.3), T(0.4) };
  generator_type generator;
  for (auto i = 0; i < 10; ++i)
    {
      auto sample = distribution(generator);
      std::cout << "sample " << i << ": ";
      std::for_each(sample.begin(),sample.end(),[](auto x){ std::cout << x << " "; });
      std::cout << std::endl;
    }
  std::cout << std::endl;
}
